package de.ozgcloud.xta.test.app.mapper;

import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.ReportingPolicy;

import genv3.de.xoev.transport.xta.x211.CodeFehlernummer;
import genv3.de.xoev.transport.xta.x211.InvalidMessageIDException;
import genv3.de.xoev.transport.xta.x211.InvalidMessageIDExceptionType;
import genv3.de.xoev.transport.xta.x211.ParameterIsNotValidException;
import genv3.de.xoev.transport.xta.x211.ParameterIsNotValidExceptionType;
import genv3.de.xoev.transport.xta.x211.XTAWSTechnicalProblemException;
import genv3.de.xoev.transport.xta.x211.XTAWSTechnicalProblemExceptionType;

@Mapper(
		unmappedTargetPolicy = ReportingPolicy.ERROR
)
public interface ExceptionMapper {

	String ERROR_CODE_LIST_URN = "urn:de:xta:webservice:codeliste:fehlernummer";
	String ERROR_CODE_LIST_VERSION = "1.0";
	String ERROR_CODE_PARAMETER_MISSING = "9020";
	String ERROR_CODE_PARAMETER_MISSING_DESCRIPTION = "Notwendige Parameter nicht vorhanden";
	String ERROR_CODE_INTERNAL_TECHNICAL_PROBLEM = "9030";
	String ERROR_CODE_INTERNAL_TECHNICAL_PROBLEM_DESCRIPTION = "Interner Fehler beim XTA-Server bzw. XTA-Dienstleister";
	String ERROR_CODE_MESSAGE_ID_UNKNOWN = "9070";
	String ERROR_CODE_MESSAGE_ID_UNKNOWN_DESCRIPTION = "MessageID für den Account nicht bekannt";

	default InvalidMessageIDException createInvalidMessageIDException(String messageId, String clientIdentifierValue) {
		return new InvalidMessageIDException(
				"No message with messageID \"%s\" available for identifier \"%s\"."
						.formatted(messageId, clientIdentifierValue),
				mapInvalidMessageIDExceptionType(ERROR_CODE_MESSAGE_ID_UNKNOWN, ERROR_CODE_MESSAGE_ID_UNKNOWN_DESCRIPTION)
		);
	}

	@Mapping(target = "errorCode", expression = "java( mapCodeFehlernummer(code, name) )")
	InvalidMessageIDExceptionType mapInvalidMessageIDExceptionType(String code, String name);

	default XTAWSTechnicalProblemException createTechnicalProblemException(String message) {
		return new XTAWSTechnicalProblemException(
				message,
				mapXTAWSTechnicalProblemExceptionType(ERROR_CODE_INTERNAL_TECHNICAL_PROBLEM, ERROR_CODE_INTERNAL_TECHNICAL_PROBLEM_DESCRIPTION)
		);
	}

	@Mapping(target = "errorCode", expression = "java( mapCodeFehlernummer(code, name) )")
	XTAWSTechnicalProblemExceptionType mapXTAWSTechnicalProblemExceptionType(String code, String name);

	default ParameterIsNotValidException createParameterIsNotValidException(String message) {
		return new ParameterIsNotValidException(
				message,
				mapParameterIsNotValidExceptionType(ERROR_CODE_PARAMETER_MISSING, ERROR_CODE_PARAMETER_MISSING_DESCRIPTION)
		);
	}

	@Mapping(target = "errorCode", expression = "java( mapCodeFehlernummer(code, name) )")
	ParameterIsNotValidExceptionType mapParameterIsNotValidExceptionType(String code, String name);

	@Mapping(target = "listURI", constant = ERROR_CODE_LIST_URN)
	@Mapping(target = "listVersionID", constant = ERROR_CODE_LIST_VERSION)
	CodeFehlernummer mapCodeFehlernummer(String code, String name);
}
