package de.ozgcloud.xta.test.app.server;

import jakarta.annotation.Generated;
import jakarta.jws.WebService;

import org.apache.cxf.annotations.SchemaValidation;
import org.apache.cxf.ws.addressing.AttributedURIType;
import org.springframework.stereotype.Component;

import de.ozgcloud.xta.test.app.mapper.ExceptionMapper;
import de.ozgcloud.xta.test.app.mapper.RequestMapper;
import de.ozgcloud.xta.test.app.mapper.ResponseMapper;
import de.ozgcloud.xta.test.app.service.XtaMessageService;
import de.ozgcloud.xta.test.app.validation.RequestValidator;
import genv3.de.xoev.transport.xta.x211.CancelDeniedException;
import genv3.de.xoev.transport.xta.x211.InvalidMessageIDException;
import genv3.de.xoev.transport.xta.x211.LookupServiceRequest;
import genv3.de.xoev.transport.xta.x211.ManagementPortType;
import genv3.de.xoev.transport.xta.x211.ParameterIsNotValidException;
import genv3.de.xoev.transport.xta.x211.PermissionDeniedException;
import genv3.de.xoev.transport.xta.x211.XTAWSTechnicalProblemException;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@WebService(serviceName = "XTAService", portName = "ManagementPort", targetNamespace = "http://xoev.de/transport/xta/211", wsdlLocation = "classpath:wsdl/XTA.wsdl", endpointInterface = "genv3.de.xoev.transport.xta.x211.ManagementPortType")
@SchemaValidation(type = SchemaValidation.SchemaValidationType.BOTH)
@RequiredArgsConstructor
@Component
@Log4j2
public class ManagementPortImpl implements ManagementPortType {

	private final ExceptionMapper exceptionMapper;
	private final ResponseMapper responseMapper;
	private final RequestMapper requestMapper;
	private final RequestValidator requestValidator;
	private final XtaMessageService xtaMessageService;

	/*
	 * (non-Javadoc)
	 * @see genv3.de.xoev.transport.xta.x211.ManagementPortType#createMessageId(genv3.eu.osci.ws.x2014.x10.transport.
	 * PartyType authorIdentifier)*
	 */
	@Override
	public AttributedURIType createMessageId(
			final genv3.eu.osci.ws.x2014.x10.transport.PartyType authorIdentifier)
			throws XTAWSTechnicalProblemException, PermissionDeniedException {
		log.info("Executing operation createMessageId");

		return responseMapper.mapCreateMessageIdResponse(
				xtaMessageService.createMessageId()
		);
	}

	/*
	 * (non-Javadoc)
	 * @see genv3.de.xoev.transport.xta.x211.ManagementPortType#checkAccountActive(genv3.eu.osci.ws.x2014.x10.transport.
	 * PartyType authorIdentifier)*
	 */
	@Override
	public void checkAccountActive(final genv3.eu.osci.ws.x2014.x10.transport.PartyType authorIdentifier)
			throws XTAWSTechnicalProblemException, PermissionDeniedException {
		log.info("Executing operation checkAccountActive");
		log.info("checkAccountActive called, author: {}", authorIdentifier);
	}

	/*
	 * (non-Javadoc)
	 * @see genv3.de.xoev.transport.xta.x211.ManagementPortType#lookupService(genv3.de.xoev.transport.xta.x211.
	 * LookupServiceRequest lookupServiceRequest, genv3.eu.osci.ws.x2014.x10.transport.PartyType authorIdentifier)*
	 */
	@Override
	@Generated(value = "org.apache.cxf.tools.wsdlto.WSDLToJava", date = "2023-03-01T15:42:39.751+01:00")
	public genv3.de.xoev.transport.xta.x211.LookupServiceResponse lookupService(
			final LookupServiceRequest lookupServiceRequest,
			final genv3.eu.osci.ws.x2014.x10.transport.PartyType authorIdentifier)
			throws XTAWSTechnicalProblemException, PermissionDeniedException, ParameterIsNotValidException {
		log.info("Executing operation lookupService");
		var request = requestMapper.mapXtaLookupServiceRequest(lookupServiceRequest, authorIdentifier);
		requestValidator.validate(request);

		return responseMapper.mapLookupServiceResponse(request);
	}

	/*
	 * (non-Javadoc)
	 * @see genv3.de.xoev.transport.xta.x211.ManagementPortType#cancelMessage(genv3.org.w3.x2005.x08.addressing.
	 * AttributedURIType messageID, genv3.eu.osci.ws.x2014.x10.transport.PartyType authorIdentifier)*
	 */
	@Override
	@Generated(value = "org.apache.cxf.tools.wsdlto.WSDLToJava", date = "2023-03-01T15:42:39.751+01:00")
	public void cancelMessage(final AttributedURIType messageID,
			final genv3.eu.osci.ws.x2014.x10.transport.PartyType authorIdentifier)
			throws XTAWSTechnicalProblemException, CancelDeniedException, PermissionDeniedException,
			ParameterIsNotValidException {
		log.info("Executing operation cancelMessage");
		throw new XTAWSTechnicalProblemException("Method not supported");
	}

	/*
	 * (non-Javadoc)
	 * @see genv3.de.xoev.transport.xta.x211.ManagementPortType#getTransportReport(genv3.org.w3.x2005.x08.addressing.
	 * AttributedURIType messageID, genv3.eu.osci.ws.x2014.x10.transport.PartyType authorIdentifier)*
	 */
	@Override
	@Generated(value = "org.apache.cxf.tools.wsdlto.WSDLToJava", date = "2023-03-01T15:42:39.751+01:00")
	public genv3.de.xoev.transport.xta.x211.TransportReport getTransportReport(
			final AttributedURIType messageID,
			final genv3.eu.osci.ws.x2014.x10.transport.PartyType authorIdentifier)
			throws XTAWSTechnicalProblemException, InvalidMessageIDException, PermissionDeniedException {
		log.info("Executing operation getTransportReport");
		var request = requestMapper.mapXtaGetTransportReportRequest(messageID, authorIdentifier);
		requestValidator.validate(request);

		var xtaTransportReport = xtaMessageService.getTransportReport(request)
				.orElseThrow(() -> exceptionMapper.createInvalidMessageIDException(request.messageId(), request.clientIdentifier().value()));

		return responseMapper.mapTransportReport(xtaTransportReport);
	}

}
