package de.ozgcloud.xta.test.app.server;

import jakarta.jws.WebService;

import org.apache.cxf.annotations.SchemaValidation;
import org.springframework.stereotype.Component;

import de.ozgcloud.xta.test.app.mapper.ExceptionMapper;
import de.ozgcloud.xta.test.app.mapper.RequestMapper;
import de.ozgcloud.xta.test.app.service.XtaMessageService;
import de.ozgcloud.xta.test.app.validation.RequestValidator;
import genv3.de.xoev.transport.xta.x211.GenericContentContainer;
import genv3.de.xoev.transport.xta.x211.MessageSchemaViolationException;
import genv3.de.xoev.transport.xta.x211.MessageVirusDetectionException;
import genv3.de.xoev.transport.xta.x211.ParameterIsNotValidException;
import genv3.de.xoev.transport.xta.x211.PermissionDeniedException;
import genv3.de.xoev.transport.xta.x211.SendPortType;
import genv3.de.xoev.transport.xta.x211.SyncAsyncException;
import genv3.de.xoev.transport.xta.x211.XTAWSTechnicalProblemException;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@WebService(serviceName = "XTAService", portName = "SendXtaPort", targetNamespace = "http://xoev.de/transport/xta/211", wsdlLocation = "classpath:wsdl/XTA.wsdl", endpointInterface = "genv3.de.xoev.transport.xta.x211.SendPortType")
@SchemaValidation(type = SchemaValidation.SchemaValidationType.BOTH)
@RequiredArgsConstructor
@Component
@Log4j2
public class SendXtaPortImpl implements SendPortType {

	private final RequestValidator validator;

	private final RequestMapper requestMapper;
	private final ExceptionMapper exceptionMapper;
	private final XtaMessageService xtaMessageService;

	/*
	 * (non-Javadoc)
	 * @see genv3.de.xoev.transport.xta.x211.SendPortType#sendMessage(genv3.de.xoev.transport.xta.x211.
	 * GenericContentContainer genericContainer, genv3.eu.osci.ws.x2014.x10.transport.MessageMetaData messageMetaData,
	 * genv3.eu.osci.ws.x2008.x05.transport.X509TokenContainerType x509TokenContainer)*
	 */
	@Override
	public void sendMessage(final GenericContentContainer genericContainer,
			final genv3.eu.osci.ws.x2014.x10.transport.MessageMetaData messageMetaData,
			final genv3.eu.osci.ws.x2008.x05.transport.X509TokenContainerType x509TokenContainer)
			throws SyncAsyncException, XTAWSTechnicalProblemException, MessageVirusDetectionException,
			MessageSchemaViolationException, PermissionDeniedException, ParameterIsNotValidException {
		log.info("Executing operation sendMessage");
		var request = requestMapper.mapXtaMessageFromGenericContentContainer(genericContainer, messageMetaData);
		validator.validate(request);

		if (!xtaMessageService.sendMessage(request)) {
			throw exceptionMapper.createParameterIsNotValidException("Failed to send message");
		}
	}

	/*
	 * (non-Javadoc)
	 * @see genv3.de.xoev.transport.xta.x211.SendPortType#sendMessageSync(genv3.de.xoev.transport.xta.x211.
	 * GenericContentContainer genericContainer, genv3.eu.osci.ws.x2014.x10.transport.MessageMetaData messageMetaData,
	 * genv3.eu.osci.ws.x2008.x05.transport.X509TokenContainerType x509TokenContainer)*
	 */
	@Override
	public void sendMessageSync(final jakarta.xml.ws.Holder<GenericContentContainer> genericContainer,
			final jakarta.xml.ws.Holder<genv3.eu.osci.ws.x2014.x10.transport.MessageMetaData> messageMetaData,
			final jakarta.xml.ws.Holder<genv3.eu.osci.ws.x2008.x05.transport.X509TokenContainerType> x509TokenContainer)
			throws SyncAsyncException, XTAWSTechnicalProblemException, MessageVirusDetectionException,
			MessageSchemaViolationException, PermissionDeniedException, ParameterIsNotValidException {
		log.info("Executing operation sendMessageSync");
		throw new UnsupportedOperationException("sendMessageSync is not supported");
	}

}
