package de.ozgcloud.xta.test.app.factory;

import static de.ozgcloud.xta.test.app.factory.PartyIdentifierTypeTestFactory.*;

import java.math.BigInteger;

import org.apache.cxf.ws.addressing.AttributedURIType;

import genv3.eu.osci.ws.x2014.x10.transport.MessageMetaData;
import genv3.eu.osci.ws.x2014.x10.transport.MsgIdentificationType;
import genv3.eu.osci.ws.x2014.x10.transport.QualifierType;

public class MessageMetaDataTestFactory {
	public static final BigInteger MESSAGE_SIZE = BigInteger.valueOf(3000);
	public static final String QUALIFER_SERVICE = "service";
	public static final String QUALIFER_SUBJECT = "subject";
	public static final String QUALIFER_MESSAGE_TYPE_CODE = "code";
	public static final String QUALIFER_MESSAGE_TYPE_PAYLOAD_SCHEMA = "payloadSchema";
	public static final String MESSAGE_ID = "de:xta:messageid:test-id";

	public static MessageMetaData create() {
		var metaData = new MessageMetaData();
		metaData.getOriginators().getAuthor().setIdentifier(AUTHOR_PARTY_IDENTIFIER);
		metaData.getDestinations().getReader().setIdentifier(READER_PARTY_IDENTIFIER);
		metaData.setMsgIdentification(createMsgIdentificationType(AttributedURITypeTestFactory.create(MESSAGE_ID)));
		metaData.setQualifier(createQualifierType(new QualifierType.BusinessScenario()));
		metaData.setMsgSize(MESSAGE_SIZE);
		return metaData;
	}

	private static MsgIdentificationType createMsgIdentificationType(AttributedURIType messageID) {
		var msgIdentificationType = new MsgIdentificationType();
		msgIdentificationType.setMessageID(messageID);
		return msgIdentificationType;
	}

	private static QualifierType createQualifierType(QualifierType.BusinessScenario businessScenario) {
		var qualifierType = new QualifierType();
		qualifierType.setService(QUALIFER_SERVICE);
		qualifierType.setSubject(QUALIFER_SUBJECT);
		qualifierType.setBusinessScenario(businessScenario);
		qualifierType.setMessageType(createMessageType());
		return qualifierType;
	}

	private static QualifierType.MessageType createMessageType() {
		var messageType = new QualifierType.MessageType();
		messageType.setCode(QUALIFER_MESSAGE_TYPE_CODE);
		messageType.setPayloadSchema(QUALIFER_MESSAGE_TYPE_PAYLOAD_SCHEMA);

		return messageType;
	}
}
