package de.ozgcloud.xta.test.app.server;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.xta.test.app.mapper.ExceptionMapper;
import de.ozgcloud.xta.test.app.mapper.RequestMapper;
import de.ozgcloud.xta.test.app.model.XtaMessage;
import de.ozgcloud.xta.test.app.service.XtaMessageService;
import de.ozgcloud.xta.test.app.validation.RequestValidator;
import genv3.de.xoev.transport.xta.x211.GenericContentContainer;
import genv3.de.xoev.transport.xta.x211.ParameterIsNotValidException;
import genv3.de.xoev.transport.xta.x211.XTAWSTechnicalProblemException;
import genv3.eu.osci.ws.x2008.x05.transport.X509TokenContainerType;
import genv3.eu.osci.ws.x2014.x10.transport.MessageMetaData;
import lombok.SneakyThrows;

public class SendXtaPortImplTest {
	@Spy
	@InjectMocks
	private SendXtaPortImpl sendXtaPortImpl;
	@Mock
	private RequestValidator requestValidator;
	@Mock
	private RequestMapper requestMapper;
	@Mock
	private XtaMessageService xtaMessageService;
	@Mock
	private ExceptionMapper exceptionMapper;

	@DisplayName("send message")
	@Nested
	class TestSendMessage {
		@Mock
		private GenericContentContainer genericContainer;
		@Mock
		private MessageMetaData messageMetaData;
		@Mock
		private X509TokenContainerType x509TokenContainer;

		private ParameterIsNotValidException parameterIsNotValidException;

		private XTAWSTechnicalProblemException technicalProblemException;

		@Mock
		private XtaMessage xtaMessage;

		@BeforeEach
		void mock() {
			parameterIsNotValidException = new ParameterIsNotValidException();
			technicalProblemException = new XTAWSTechnicalProblemException();
			when(requestMapper.mapXtaMessageFromGenericContentContainer(genericContainer, messageMetaData)).thenReturn(xtaMessage);
		}

		@DisplayName("should return")
		@Test
		@SneakyThrows
		void shouldSendMessage() {
			when(xtaMessageService.sendMessage(xtaMessage)).thenReturn(true);

			sendMessage();
		}

		@DisplayName("should throw if send fails")
		@Test
		void shouldThrowIfSendFails() {
			when(xtaMessageService.sendMessage(xtaMessage)).thenReturn(false);
			when(exceptionMapper.createParameterIsNotValidException(any())).thenReturn(parameterIsNotValidException);

			assertThatThrownBy(this::sendMessage)
					.isEqualTo(parameterIsNotValidException);
		}

		@DisplayName("should validate request")
		@Test
		@SneakyThrows
		void shouldValidateRequest() {
			doThrow(technicalProblemException).when(requestValidator).validate(xtaMessage);

			assertThatThrownBy(this::sendMessage)
					.isEqualTo(technicalProblemException);
		}

		@SneakyThrows
		private void sendMessage() {
			sendXtaPortImpl.sendMessage(genericContainer, messageMetaData, x509TokenContainer);
		}

	}

}
