package de.ozgcloud.xta.test.app.validation;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.xta.test.app.factory.XtaGetMessageRequestTestFactory;
import de.ozgcloud.xta.test.app.mapper.ExceptionMapper;
import de.ozgcloud.xta.test.app.model.XtaIdentifier;
import genv3.de.xoev.transport.xta.x211.XTAWSTechnicalProblemException;
import lombok.SneakyThrows;

class RequestValidatorTest {

	@InjectMocks
	private RequestValidator validator;

	@Mock
	private ExceptionMapper exceptionMapper;

	@Mock
	private XTAWSTechnicalProblemException exception;

	@DisplayName("validate xta get message request")
	@Nested
	class TestValidateXtaGetMessageRequest {

		@DisplayName("should return")
		@Test
		@SneakyThrows
		void shouldReturn() {
			validator.validate(XtaGetMessageRequestTestFactory.createBuilder()
					.clientIdentifier(XtaIdentifier.builder()
							.value("not blank")
							.build())
					.build());
		}

		@DisplayName("should throw if null")
		@Test
		void shouldThrowIfNull() {
			when(exceptionMapper.createTechnicalProblemException(any())).thenReturn(exception);

			assertThatThrownBy(() -> validator.validate(null)).isEqualTo(exception);
		}

		@DisplayName("with invalid request")
		@Nested
		class TestWithInvalidRequest {

			@BeforeEach
			void mock() {
				when(exceptionMapper.createTechnicalProblemException(any())).thenReturn(exception);
			}

			@DisplayName("should throw if message id is null")
			@Test
			void shouldThrowIfMessageIdIsNull() {
				var request = XtaGetMessageRequestTestFactory.createBuilder()
						.messageId(null)
						.build();

				assertThatThrownBy(() -> validator.validate(request)).isEqualTo(exception);
			}

			@DisplayName("should throw if message id does not match pattern")
			@Test
			void shouldThrowIfMessageIdDoesNotMatchPattern() {
				var request = XtaGetMessageRequestTestFactory.createBuilder()
						.messageId("urn:unknown-namespace:application:000ca2fe-f4e1-45c2-8233-3a0eb760bd16")
						.build();

				assertThatThrownBy(() -> validator.validate(request)).isEqualTo(exception);
			}

			@DisplayName("should throw if clientIdentifier value is null")
			@Test
			void shouldThrowIfClientIdentifierValueIsNull() {
				var request = XtaGetMessageRequestTestFactory.createBuilder()
						.clientIdentifier(XtaIdentifier.builder()
								.value(null)
								.build())
						.build();

				assertThatThrownBy(() -> validator.validate(request)).isEqualTo(exception);
			}
		}

	}

}